map = {}
map.data = {}
map.canvas = {}
map.canvas.canvas = nil
map.canvas.x = 0
map.canvas.y = 0
map.canvas.width = 0
map.canvas.height = 0
map.background = {}
map.background[1] = {}
map.background[2] = {}
map.background[3] = {}


-- tries to load map with the given name
function map.load( mapname )
    
    
    if love.filesystem.exists(mapname..".map") then
        
        -- load the placement for tiles
		map.data = Tserial.unpack(love.filesystem.read(mapname..".map"))
        
        -- load the tileset
        tileset.load(mapname.."/tileset.png")
        
        -- calculate required canvas width and height for map
        map.canvas.width = tileset.getWidth() * map.data.tilesx * window.scale
        map.canvas.height = tileset.getHeight() * map.data.tilesy * window.scale
        
        -- create canvas for map
        map.canvas.canvas = love.graphics.newCanvas(map.canvas.width, map.canvas.height)
        
        -- load backgrounds
        map.background[1].image = love.graphics.newImage("gfx/"..mapname.."/"..map.data.background[1].image)
        map.background[2].image = love.graphics.newImage("gfx/"..mapname.."/"..map.data.background[2].image)
        map.background[3].image = love.graphics.newImage("gfx/"..mapname.."/"..map.data.background[3].image)
        
        -- create canvases for backgrounds
        map.background[1].canvas = love.graphics.newCanvas(map.background[1].image:getDimensions())
        map.background[2].canvas = love.graphics.newCanvas(map.background[2].image:getDimensions())
        map.background[3].canvas = love.graphics.newCanvas(map.background[3].image:getDimensions())
    
    else
        
        love.graphics.print("Map file does not exist.",0,10)
        
    end
    
    
end


function map.save(mapname)
    
		love.filesystem.write(mapname..".map", Tserial.pack(map.data))
    
end


function map.draw()
    
    
    -- draw background canvas(es)
    love.graphics.setCanvas(map.background[1].canvas)
    
        love.graphics.draw(map.background[1].image, map.data.background[1].x, map.data.background[1].y)
    
    love.graphics.setCanvas(map.background[2].canvas)
    
        love.graphics.draw(map.background[2].image, map.data.background[2].x, map.data.background[2].y)
    
    love.graphics.setCanvas(map.background[3].canvas)
    
        love.graphics.draw(map.background[3].image, map.data.background[3].x, map.data.background[3].y)
    
    
    -- draw map canvas
    love.graphics.setCanvas(map.canvas.canvas)         -- set the map canvas as render target
    love.graphics.push()
    love.graphics.scale(window.scale)
    
    for y, i in ipairs(map.data) do
        for x, h in ipairs(i) do
            
            love.graphics.draw(tileset.image, tileset.quads[h], (x-1)*tileset.tile.width, (y-1)*tileset.tile.height)
            
        end
    end
    
    love.graphics.pop()
    love.graphics.setCanvas()                   -- reset render target
    
    
end


function map.createBlankMap(sizex,sizey)
    
    
    -- actual map
    for y = 1, sizey, 1 do
        
        map.data[y] = {}
        
        for x = 1, sizex, 1 do
            map.data[y][x] = 0
        end
        
    end
    
    map.data.tilesx = sizex
    map.data.tilesy = sizey
    
    -- decorational background layers
    map.data.background = {}
    map.data.background[1] = {}
    map.data.background[1].x = 0
    map.data.background[1].y = 0
    map.data.background[1].scrollshare = 0
    map.data.background[1].image = "background.png"
    map.data.background[2] = {}
    map.data.background[2].x = 0
    map.data.background[2].y = 0
    map.data.background[2].scrollshare = 0
    map.data.background[2].image = "transparent.png"
    map.data.background[3] = {}
    map.data.background[3].x = 0
    map.data.background[3].y = 0
    map.data.background[3].scrollshare = 0
    map.data.background[3].image = "transparent.png"
    
    map.save("level1")
    
    
end